﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace SADC_Decrypt
{
    class Program
    {
        [STAThread]
        static void Main(string[] args)
        {
            string inputPath = null, outputPath = null;
            if (args.Length > 0)
            {
                if (File.Exists(args[0]))
                {
                    inputPath = args[0];
                }
                if (args.Length > 1)
                {
                    string dir = Path.GetDirectoryName(Path.GetFullPath(args[1]));
                    if (Directory.Exists(dir))
                    {
                        outputPath = args[1];
                    }
                }
            }

            if (inputPath == null)
            {
                OpenFileDialog ofd = new OpenFileDialog();
                ofd.Title = "Open VMS file";
                ofd.Filter = "Dreamcast VMU saves (*.vms)|*.vms";
                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    inputPath = ofd.FileName;
                }
            }
            if(inputPath != null){
                byte[] vmsFile = File.ReadAllBytes(inputPath);
                ProcessVMS(ref vmsFile);

                if (outputPath == null)
                {
                    SaveFileDialog sfd = new SaveFileDialog();
                    sfd.Title = "Save VMS file";
                    sfd.Filter = "Dreamcast VMU saves (*.vms)|*.vms";
                    if (sfd.ShowDialog() == DialogResult.OK)
                    {
                        outputPath = sfd.FileName;
                    }
                }
                if(outputPath != null){
                    File.WriteAllBytes(outputPath, vmsFile);
                }
            }
        }

        public static void ProcessVMS(ref byte[] input)
        {
            int aniFrames = input[0x40] + (input[0x41] << 8);
            int dataStart = 0x80 + (aniFrames * 0x200);
            byte[] encrypted = new byte[input.Length - dataStart];
            Array.Copy(input, dataStart, encrypted, 0, encrypted.Length);
            DecryptData(ref encrypted);
            Array.Copy(encrypted, 0, input, dataStart, encrypted.Length);
        }

        public static void DecryptData(ref byte[] input)
        {
            //Translated from some perl code someone posted by Darksecond on AssemblerGames
			//Original post here: http://assemblergames.com/l/threads/decoding-decrypting-sonic-adventure-ranking-data.60036/#post-863289
            byte[] xor_code = new byte[] { 0x41, 0x54, 0x45, 0x5A };
            byte[] plus_val = new byte[] { 0x41, 0x4E, 0x41, 0x4E };
            byte[] tmp_val = new byte[] { 0, 0, 0, 0 };

            for (int i = 0; i < input.Length / 4; i++)
            {
                int tmp_val2 = 0;
                for (int j = 0; j < 4; j++)
                {
                    input[i * 4 + j] = (byte)(input[i*4+j] ^  tmp_val[j] ^ xor_code[j]);
                    int next = (tmp_val[j] + plus_val[j] + tmp_val2);
                    tmp_val2 = (next >> 8);
                    tmp_val[j] = (byte)(next & 0xFF);
                }
            }
        }
    }
}
